# Generated by Django 5.1.6 on 2025-08-11 04:47

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('saas_base', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='member',
            old_name='invite_email',
            new_name='email',
        ),
        migrations.AddField(
            model_name='member',
            name='name',
            field=models.CharField(blank=True, max_length=300, null=True, verbose_name='name'),
        ),
        migrations.AddField(
            model_name='member',
            name='is_staff',
            field=models.BooleanField(default=True),
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('name', models.CharField(max_length=100, primary_key=True, serialize=False)),
                ('description', models.CharField(blank=True, max_length=480)),
                ('created_at', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('permissions', models.ManyToManyField(blank=True, to='saas_base.permission')),
            ],
            options={
                'verbose_name': 'role',
                'verbose_name_plural': 'roles',
                'db_table': 'saas_role',
                'ordering': ['created_at'],
            },
        ),
        migrations.AddField(
            model_name='member',
            name='role',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=models.SET_NULL,
                to='saas_base.role',
            ),
        ),
    ]
