# Generated by Django 6.0.1 on 2026-01-11 05:52

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('saas_base', '0004_tenant_logo'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='profile', serialize=False, to=settings.AUTH_USER_MODEL)),
                ('bio', models.TextField(blank=True, max_length=500)),
                ('birth_date', models.DateField(blank=True, null=True)),
                ('website', models.URLField(blank=True, null=True)),
                ('avatar_url', models.URLField(blank=True, null=True)),
                ('region', models.CharField(blank=True, max_length=4, null=True)),
                ('locale', models.CharField(blank=True, max_length=10, null=True)),
                ('timezone', models.CharField(default='UTC', help_text="e.g., 'America/New_York'", max_length=50)),
            ],
            options={
                'db_table': 'saas_user_profile',
            },
        ),
        migrations.AlterModelTable(
            name='useremail',
            table='saas_user_email',
        ),
    ]
