<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width">
  <title>{{ error.title }}</title>
  <style>
    body {
      margin: 0;
      padding: 0;
      display: flex;
      align-items: center;
      justify-content: center;
      font-size: 16px;
      font-family: system-ui,"Segoe UI",Roboto,Helvetica,Arial,sans-serif;
    }
    .box {
      display: flex;
      align-items: center;
      color: #686869;
    }
    .box h1 {
      font-size: 1.2rem;
      line-height: 2;
      border-right: 1px solid #ddd;
      padding-right: 1rem;
      margin-right: 1rem;
      color: #121211;
    }
  </style>
</head>
<body>
<div class="box">
  <h1>{{ error.code }}</h1>
  <div>{{ error.message }}</div>
</div>
</body>
</html>
