{% extends "saas_emails/base.html" %}

{% block title %}Add email{% endblock %}

{% block content %}
  <p>Hello {{ user }},</p>

  <p>We received a request to link this email to your account. To verify ownership, please use the following authorization code:</p>

  <p>Your Authorization Code:</p>

  {% include "saas_emails/partials/auth_code.html" %}

  <p>If you didn't make this request, you can safely ignore this email. Your account remains secure.</p>
{% endblock %}
